@@ -12,24 +12,8 @@ using namespace std;
 #include "nwcclex.h"
 
 
-// lexical analyzer             (C) H.Niwa 1993-2009
 
-/*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
-
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
- */
 
 
 #include "config.h"
@@ -43,6 +27,7 @@ using namespace std;
 #include <libgen.h>
 #include <setjmp.h>
 #include <getopt.h>
+#include <signal.h>
 
 #include <complex>
 
@@ -58,7 +43,15 @@ using namespace std;
 #include "builtin.h"
 #include "sysmodule.h"
 
-char *VERSION = "0.8.0";
+const char *VERSION = "0.17.0";
+void
+optver ()
+{
+  extern std::string code;
+  printf ("Descartes %s\t(code:%s) \n", VERSION, code.c_str ());
+  printf ("Copyright (C) 2009 - 2010 Hideyuki Niwa\n\n");
+}
+
 
 extern int pllexSpace (int c);
 extern int pllexCkSpace (int c);
@@ -69,10 +62,12 @@ int Char;
 
 int ErrorFlg = 0;
 FILE *RdFp, *pr;
-char *MESG = NULL;
+const char *MESG = NULL;
 
 int EndFlg = 0;
 
+int resultflg = 1;
+
 void
 opthelp ()
 {
@@ -85,16 +80,62 @@ opthelp ()
   printf (" -e, --euc\tEUC code is used\n");
   printf (" -s, --sjis\tSJIS code is used\n");
   printf ("\n");
+  printf (" -t, --trace\ttrace becomes effective\n");
+  printf (" -r, --result\tresult is not displayed\n");
+  printf ("\n");
 }
 
 void
-optver ()
+sigint_handler_cmdline (int signum)
 {
-  extern std::string code;
-  printf ("Descartes %s\t(code:%s) \n", VERSION, code.c_str ());
-  printf ("Copyright (C) 2009 Hideyuki Niwa\n\n");
+  int i;
+  char buf[80];
+  int c;
+
+  printf ("\nquit ? [y or n] ");
+  if (fgets (buf, 80 - 1, stdin) == NULL)
+    {
+      return;
+    }
+
+  for (i = 0; i < 80; i++)
+    {
+      c = buf[i];
+      if (c == ' ')
+	{
+	  continue;
+	}
+      else if ((c == 'y') || (c == 'Y'))
+	{
+	  exit (0);
+	}
+      else
+	{
+	  break;
+	}
+    }
 }
 
+#ifdef __MINGW32__
+void
+set_sigint_handler_cmdline ()
+{
+  signal (SIGINT, sigint_handler_cmdline);
+}
+
+#else
+struct sigaction sigint_action;
+
+void
+set_sigint_handler_cmdline ()
+{
+  sigint_action.sa_handler = sigint_handler_cmdline;
+  sigemptyset (&(sigint_action.sa_mask));
+  sigint_action.sa_flags = 0;
+  sigaction (SIGINT, &sigint_action, NULL);
+}
+
+#endif
 
 int pargc;
 char **pargv;
@@ -119,6 +160,8 @@ main (int argc, char **argv)
     {"utf8", 0, 0, 'u'},
     {"euc", 0, 0, 'e'},
     {"sjis", 0, 0, 's'},
+    {"trace", 0, 0, 't'},
+    {"result", 0, 0, 'r'},
     {0, 0, 0, 0}
   };
 
@@ -131,7 +174,8 @@ main (int argc, char **argv)
       int this_option_optind = optind ? optind : 1;
       int option_index = 0;
 
-      optc = getopt_long (argc, argv, "h?vues", long_options, &option_index);
+      optc = getopt_long (argc, argv, "h?vuestr",
+			  long_options, &option_index);
       if (optc == -1)
 	{
 	  break;
@@ -156,6 +200,12 @@ main (int argc, char **argv)
 	case 's':
 	  code = "SJIS";
 	  break;
+	case 't':
+	  Tron ();
+	  break;
+	case 'r':
+	  resultflg = 0;
+	  break;
 	default:
 	  opthelp ();
 	  exit (0);
@@ -166,13 +216,11 @@ main (int argc, char **argv)
 #ifdef __CYGWIN__
   char inputrcpath[MAXPATHLEN + 10];
 
-  // set INPUTRC path for readline
   char *homepath = getenv ("HOME");
   strncpy (inputrcpath, homepath, MAXPATHLEN);
   strcat (inputrcpath, "/inputrc");
   setenv ("INPUTRC", inputrcpath, 0);
-//printf("inputrcpath %s \n", inputrcpath);
-#endif /* __CYGWIN__ */
+#endif
 
 #ifdef __MINGW32__
   extern std::string code;
@@ -190,7 +238,7 @@ main (int argc, char **argv)
   struct timeval tv;
   gettimeofday (&tv, NULL);
   srandom (tv.tv_usec);
-#endif /* __MINGW32__ */
+#endif
 
   extern void StackBase ();
   StackBase ();
@@ -199,12 +247,20 @@ main (int argc, char **argv)
   GetTmpPath ();
   GetEditorPath ();
 
+#if 0
+  Context *cx = new Context (Module);
+  IncludeSub (cx, Nil, Module, "sys");
+  delete cx;
+#endif
+
   if (argc <= optind)
     {
       DResultFlag = 1;
 
       optver ();
 
+      set_sigint_handler_cmdline ();
+
       GetlineEval ();
 
       exit (0);
@@ -220,16 +276,13 @@ main (int argc, char **argv)
       free (dargv1);
       free (bargv1);
 
-//printf("fopen 0 argv[optind] %s dname %s bname %s \n", argv[optind], dname, bname);
       if ((dname[0] == '.') && (dname[1] == 0) && ((argv[optind])[1] != '/'))
 	{
 	  Node *nd;
 	  RdFp = NULL;
 
-//PrintNode("dlibpathnode ", dlibpathnode);
 	  if (dlibpathnode == Nil)
 	    {
-//printf("fopen 1 argv[optind] %s \n", argv[optind]);
 	      RdFp = fopen (argv[optind], "rb");
 	    }
 	  else
@@ -256,7 +309,6 @@ main (int argc, char **argv)
 	}
       else
 	{
-//printf("fopen 2 argv[optind] %s \n", argv[optind]);
 	  RdFp = fopen (argv[optind], "rb");
 	}
 
@@ -277,13 +329,13 @@ main (int argc, char **argv)
 	  RdFp = NULL;
 	}
 
-//              fflush(stdout); PPmodule();
 
       exit (0);
     }
 }
 
 
+
 long
 FilePointSave ()
 {
@@ -301,17 +353,33 @@ SkipComment (int &Char)
 {
   int c;
 
+  if (Char == EOF)
+    {
+      return;
+    }
+
   for (;;)
     {
-      // #! comment
+      if (Char == EOF)
+	{
+	  break;
+	}
       if (Char == '#')
 	{
 	  c = fgetc (RdFp);
+	  if (c == EOF)
+	    {
+	      break;
+	    }
 	  if (c == '!')
 	    {
 	      for (;;)
 		{
 		  c = fgetc (RdFp);
+		  if (c == EOF)
+		    {
+		      break;
+		    }
 		  if (c == '\n')
 		    {
 		      break;
@@ -324,16 +392,23 @@ SkipComment (int &Char)
 	      ungetc (c, RdFp);
 	      return;
 	    }
-	  // '//' and '/* */' comment
 	}
       else if (Char == '/')
 	{
 	  c = fgetc (RdFp);
+	  if (c == EOF)
+	    {
+	      break;
+	    }
 	  if (c == '/')
-	    {			// comment
+	    {
 	      for (;;)
 		{
 		  c = fgetc (RdFp);
+		  if (c == EOF)
+		    {
+		      break;
+		    }
 		  if (c == '\n')
 		    {
 		      break;
@@ -342,13 +417,21 @@ SkipComment (int &Char)
 	      Char = fgetc (RdFp);
 	    }
 	  else if (c == '*')
-	    {			/* comment */
+	    {
 	      for (;;)
 		{
 		  c = fgetc (RdFp);
+		  if (c == EOF)
+		    {
+		      break;
+		    }
 		  if (c == '*')
-		    {		/* close comment */
+		    {
 		      c = fgetc (RdFp);
+		      if (c == EOF)
+			{
+			  break;
+			}
 		      if (c == '/')
 			{
 			  break;
@@ -366,6 +449,10 @@ SkipComment (int &Char)
 			  Char = '/';
 			  SkipComment (Char);
 			}
+		      if (c == EOF)
+			{
+			  break;
+			}
 		    }
 		}
 	      Char = fgetc (RdFp);
@@ -424,7 +511,7 @@ SkipSpace ()
 }
 
 int
-CHECK (char *Pt)
+CHECK (const char *Pt)
 {
   long TelSav;
   int r;
@@ -437,7 +524,7 @@ CHECK (char *Pt)
 
 
 int
-NotSkipSpCOMP (char *Pt)
+NotSkipSpCOMP (const char *Pt)
 {
   long TelSav;
   int c;
@@ -465,7 +552,7 @@ NotSkipSpCOMP (char *Pt)
 }
 
 int
-COMP (char *Pt)
+COMP (const char *Pt)
 {
   long TelSav;
   int r, c;
@@ -529,7 +616,7 @@ static char errorbuf[1024];
 int ErrorStopFlag = 0;
 
 void
-ErrorStop (char *fmt, ...)
+ErrorStop (const char *fmt, ...)
 {
   va_list argptr;
   int LineNo = 1;
@@ -555,7 +642,6 @@ ErrorStop (char *fmt, ...)
 
   fprintf (stderr, "# %d : error... %s\n", LineNo, errorbuf);
 
-//      fclose(pr);
   ErrorFlg = -1;
   ErrorStopFlag = 1;
 
@@ -567,9 +653,9 @@ ErrorStop (char *fmt, ...)
 }
 
 void
-UNTIL (char *s)
+UNTIL (const char *s)
 {
-  char *Pt;
+  const char *Pt;
   int c;
   int matchFlg = 0;
   long tellsave;
@@ -603,9 +689,9 @@ UNTIL (char *s)
 }
 
 void
-SKIP (char *s)
+SKIP (const char *s)
 {
-  char *Pt;
+  const char *Pt;
   int c;
   int matchFlg = 0;
 
@@ -632,7 +718,7 @@ SKIP (char *s)
 
 
 int
-NotSkipSpNOT (char *s)
+NotSkipSpNOT (const char *s)
 {
   long TelSav;
   int rc;
@@ -644,7 +730,7 @@ NotSkipSpNOT (char *s)
 }
 
 int
-NOT (char *s)
+NOT (const char *s)
 {
   long TelSav;
   int rc;
@@ -768,7 +854,7 @@ SPACE ()
 void
 SetToken (int c)
 {
-  if (tokenPt >= TOKEN_LEN)
+  if (tokenPt >= TOKEN_LEN - 3)
     {
       ErrorStop ("Too long token ");
     }
@@ -880,14 +966,12 @@ STRINGS ()
       return -1;
     }
   qt = Char;
-//      SetToken(Char);
 
   for (;;)
     {
       C ();
       if (Char == qt)
 	{
-//                      SetToken(Char);
 	  SetToken (0);
 	  return 0;
 	}
