@@ -1,5 +1,5 @@
 /*
- * token operation program copyright (C) 2009 H.Niwa
+ * token operation program copyright (C) 2009 - 2010 H.Niwa
  */
 
 /*
@@ -63,7 +63,9 @@ int tokenStrings(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenWildcard(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenWORD(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
+int tokenSNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenFNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
+int tokenSFNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenID(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenRange(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals,
 							List* module);
@@ -79,6 +81,8 @@ int tokenNextStr(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenNotNextStr(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenNextSyntax(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 int tokenNotNextSyntax(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
+int tokenCountIndent(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
+int tokenNextIndent(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals);
 
 std::string	Tokenbuf = "";
 
@@ -590,9 +594,25 @@ int tokenCR(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 		return 0;
 	}
 
-	int c = fgetc(cx->ioin);
+	long fpos = ftell(cx->ioin);
+
+	int c;
+	for( ; c != EOF; ) {
+		c = fgetc(cx->ioin);
+		if (c == '\t') {
+			continue;
+		}
+		if ((c == '\n') || (c == EOF)) {
+			break;
+		}
+		if (!isSpace(cx, c)) {
+			break;
+		}
+	}
+
 	if (c != '\n') {
-		ungetc(c, cx->ioin);
+//		ungetc(c, cx->ioin);
+		fseek(cx->ioin, fpos, SEEK_SET);
 		return -1;
 	}
 
@@ -1055,6 +1075,62 @@ int tokenNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 	return -1;
 }
 
+int tokenSNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
+{
+	if (cx->ioin == stdin) {
+		syserr("File is not opened");
+		return 0;
+	}
+
+	int c;
+	
+	int ll = ListLength(goalscar);
+	if ((ll != 2) && (ll != 1)) {
+		syserr("usage : <SNUM [VAR]> \n");
+		return 0;
+	}
+
+	if ((ll == 2) && (goalscar->Cdr()->Car()->kind() != UNDEF)) {
+		syserr("usage : <SNUM [VAR]> \n");
+		return 0;
+	}
+
+	Tokenbuf = "";
+	
+	tokenSkipSpace(cx);
+
+	c = fgetc(cx->ioin);
+	if ((c == '+') || (c == '-')) {
+		Tokenbuf = Tokenbuf + (char)c;
+	} else {
+		ungetc(c, cx->ioin);
+	}
+	
+	for (;;) {
+		c = fgetc(cx->ioin);
+
+		if (isdigit(c)) {
+		 	Tokenbuf = Tokenbuf + (char)c;
+		} else {
+			ungetc(c, cx->ioin);
+			
+			if (Tokenbuf == "") {
+				break;
+			}
+			
+			Node* env=Nil->Cons(Nil);
+			PushStack(cx, goalscar, Nil, env);
+
+			if (ll == 1) {
+				return 1;
+			}
+			return TokenVar(cx, goalscar);
+		}
+	}
+	Tokenbuf = "";
+	return -1;
+}
+
 int tokenFNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 {
 	if (cx->ioin == stdin) {
@@ -1147,6 +1223,110 @@ int tokenFNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 
 }
 
+int tokenSFNum(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
+{
+	if (cx->ioin == stdin) {
+		syserr("File is not opened");
+		return 0;
+	}
+
+	int ll = ListLength(goalscar);
+	if ((ll != 2) && (ll != 1)) {
+		syserr("usage : <SFNUM [VAR]> \n");
+		return 0;
+	}
+
+	if ((ll == 2) && (goalscar->Cdr()->Car()->kind() != UNDEF)) {
+		syserr("usage : <SFNUM [VAR]> \n");
+		return 0;
+	}
+
+	int	dot = 0;
+
+	tokenSkipSpace(cx);
+
+	Tokenbuf = "";
+	int c;
+	
+	c = fgetc(cx->ioin);
+	if ((c == '+') || (c == '-')) {
+		Tokenbuf = Tokenbuf + (char)c;
+	} else {
+		ungetc(c, cx->ioin);
+	}
+	
+	c = fgetc(cx->ioin);
+	if (!isdigit(c) && (c != '.')) {
+		if (Tokenbuf != "") {
+			ungetc(c, cx->ioin);
+		}
+		Tokenbuf = "";
+		ungetc(c, cx->ioin);
+		return -1;
+	}
+	if (c == '.') {
+		dot++;
+	}
+	Tokenbuf = Tokenbuf + (char)c;
+
+	for (;;) {
+		c = fgetc(cx->ioin);
+		if (c == '.') {
+			if (dot) {
+				ungetc(c, cx->ioin);
+				int i;
+				for (i = Tokenbuf.length()-1; i >= 0; i--) {
+					ungetc(Tokenbuf.c_str()[i], cx->ioin);
+				}
+				Tokenbuf = "";
+				return -1;
+			}
+			dot++;
+			Tokenbuf = Tokenbuf + (char)c;
+		} else if ((c == 'e') || (c == 'E')) {
+			Tokenbuf = Tokenbuf + 'E';
+			c = fgetc(cx->ioin);
+			if (isdigit(c) || (c == '+') || (c == '-')) {
+				Tokenbuf = Tokenbuf + (char)c;
+				for (;;) {
+					c = fgetc(cx->ioin);
+					if (!isdigit(c)) {
+						ungetc(c, cx->ioin);
+
+						if (ll == 1) {
+							return 1;
+						}
+						return TokenVar(cx, goalscar);
+					}
+					Tokenbuf = Tokenbuf + (char)c;
+				}
+			} else {
+				int i;
+				ungetc(c, cx->ioin);
+				for (i = Tokenbuf.length()-1; i >= 0; i--) {
+					ungetc(Tokenbuf.c_str()[i], cx->ioin);
+				}
+				Tokenbuf = "";
+				return -1;
+			}
+		} else if (!isdigit(c)) {
+			ungetc(c, cx->ioin);
+
+			Node* env=Nil->Cons(Nil);
+			PushStack(cx, goalscar, Nil, env);
+
+			if (ll == 1) {
+				return 1;
+			}
+			return TokenVar(cx, goalscar);
+		} else {
+			Tokenbuf = Tokenbuf + (char)c;
+		}
+	}
+	return -1;
+
+}
+
 
 int tokenID(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 {
@@ -1551,6 +1731,7 @@ int tokenNullLine(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
 	if ((c == EOF) || (c == '\n')) {
 		return 1;
 	} 
+	ungetc(c, cx->ioin);
 	return -1;
 }
 
@@ -1828,5 +2009,180 @@ int tokenNotNextSyntax(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goal
 	return rn;
 }
 
+int tokenCountIndent(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
+{
+	if (cx->ioin == stdin) {
+		syserr("File is not opened");
+		return 0;
+	}
+
+	int ll = ListLength(goalscar);
+	if (ll != 2) {
+		syserr("usage : <indent VAR> or <indent NUM>\n");
+		return 0;
+	}
+
+	Node* nvar = goalscar->Cdr()->Car()->Val();
+
+	if ((nvar->kind() != UNDEF) && (nvar->kind() != ATOM)) {
+		syserr("usage : <indent VAR> or <indent NUM>\n");
+		return 0;
+	}
+
+	
+	long fpo = ftell(cx->ioin);
+
+	int i, c;
+	for (i = fpo-1; i >= 0; i--) {
+		fseek(cx->ioin, i, 0);
+		if (i == 0) {
+			break;
+		}
+		c = fgetc(cx->ioin);
+		if (c == EOF) {
+			break;
+		}
+		if (c == '\n') {
+			break;
+		}
+	}
+	char* spc = CharSpace();
+	long long n;
+	for (n = 0; ; n++) {
+		c = fgetc(cx->ioin);
+		if (c == '\t') {
+			n = n + 8;
+		}
+		
+		if (c == (spc[0] & 0xff)) {
+			int j, c2, flg=1;
+			for (j = 1; j < strlen(spc); j++) {
+				c2 = fgetc(cx->ioin);
+				if (c2 != (spc[j] & 0xff)) {
+					flg = 0;
+					break;
+				}
+			}
+			if (flg) {
+				n = n + 1;
+			} else {
+				break;
+			}
+		} else if (c != ' ') {
+			break;
+		}
+	}	
+	fseek(cx->ioin, fpo, 0);
+
+	if (nvar->kind() == ATOM) {
+		long long num;
+
+		if (!((Atom*)nvar)->toInt(num)) {
+			return -1;
+		}
+
+		if (num == n) {
+			return 1;
+		} else {
+			return -1;
+		}
+	}
+
+	Node* env=Nil->Cons(Nil);
+	PushStack(cx, goalscar, Nil, env);
+
+	SetEnv(env, nvar);
+	((Undef*)nvar)->Set(mka(n));
+
+	PushStack(cx, Nil, Nil, env);
+	
+	return 1;
+}
+
+
+int tokenNextIndent(Context* cx, Node* goalscar, Node* &goalscdr, Node* &goals)
+{
+	if (cx->ioin == stdin) {
+		syserr("File is not opened");
+		return 0;
+	}
+
+	int ll = ListLength(goalscar);
+	if (ll != 2) {
+		syserr("usage : <NEXTINDENT VAR> or <NEXTINDENT NUM>\n");
+		return 0;
+	}
+
+	Node* nvar = goalscar->Cdr()->Car()->Val();
+
+	if ((nvar->kind() != UNDEF) && (nvar->kind() != ATOM)) {
+		syserr("usage : <NEXTINDENT VAR> or <NEXTINDENT NUM>\n");
+		return 0;
+	}
+
+	int	c;	
+	long fpo = ftell(cx->ioin);
+
+	for (;;) {
+		c = fgetc(cx->ioin);
+		if (c == '\n') {
+			break;
+		} else if (c == EOF) {
+			break;
+		}
+	}
+
+	char* spc = CharSpace();
+	long long n;
+	for (n = 0; ; n++) {
+		c = fgetc(cx->ioin);
+		if (c == '\t') {
+			n = n + 8;
+		}
+		
+		if (c == (spc[0] & 0xff)) {
+			int j, c2, flg=1;
+			for (j = 1; j < strlen(spc); j++) {
+				c2 = fgetc(cx->ioin);
+				if (c2 != (spc[j] & 0xff)) {
+					flg = 0;
+					break;
+				}
+			}
+			if (flg) {
+				n = n + 1;
+			} else {
+				break;
+			}
+		} else if (c != ' ') {
+			break;
+		}
+	}	
+	fseek(cx->ioin, fpo, 0);
+
+	if (nvar->kind() == ATOM) {
+		long long num;
+
+		if (!((Atom*)nvar)->toInt(num)) {
+			return -1;
+		}
+
+		if (num == n) {
+			return 1;
+		} else {
+			return -1;
+		}
+	}
+
+	Node* env=Nil->Cons(Nil);
+	PushStack(cx, goalscar, Nil, env);
+
+	SetEnv(env, nvar);
+	((Undef*)nvar)->Set(mka(n));
+
+	PushStack(cx, Nil, Nil, env);
+	
+	return 1;
+}
 
 
